@@ -324,7 +324,6 @@ static int _lvstatus_disp(struct report_handle *rh, struct field *field,
 	const struct logical_volume *lv = (const struct logical_volume *) data;
 	struct lvinfo info;
 	char *repstr;
-	struct lv_segment *snap_seg;
 	float snap_percent;
 
 	if (!(repstr = dm_pool_zalloc(rh->mem, 7))) {
@@ -334,9 +333,12 @@ static int _lvstatus_disp(struct report_handle *rh, struct field *field,
 
 	if (lv->status & PVMOVE)
 		repstr[0] = 'p';
-	else if (lv->status & MIRRORED)
-		repstr[0] = 'm';
-	else if (lv->status & MIRROR_IMAGE)
+	else if (lv->status & MIRRORED) {
+		if (lv->status & MIRROR_NOTSYNCED)
+			repstr[0] = 'M';
+		else
+			repstr[0] = 'm';
+	}else if (lv->status & MIRROR_IMAGE)
 		repstr[0] = 'i';
 	else if (lv->status & MIRROR_LOG)
 		repstr[0] = 'l';
@@ -344,7 +346,7 @@ static int _lvstatus_disp(struct report_handle *rh, struct field *field,
 		repstr[0] = 'v';
 	else if (lv_is_origin(lv))
 		repstr[0] = 'o';
-	else if (find_cow(lv))
+	else if (lv_is_cow(lv))
 		repstr[0] = 's';
 	else
 		repstr[0] = '-';
@@ -377,8 +379,8 @@ static int _lvstatus_disp(struct report_handle *rh, struct field *field,
 			repstr[4] = 'd';	/* Inactive without table */
 
 		/* Snapshot dropped? */
-		if (info.live_table && (snap_seg = find_cow(lv)) &&
-		    (!lv_snapshot_percent(snap_seg->cow, &snap_percent) ||
+		if (info.live_table && lv_is_cow(lv) &&
+		    (!lv_snapshot_percent(lv, &snap_percent) ||
 		     snap_percent < 0 || snap_percent >= 100)) {
 			repstr[0] = toupper(repstr[0]);
 			if (info.suspended)
@@ -473,7 +475,8 @@ static int _vgstatus_disp(struct report_handle *rh, struct field *field,
 	return 1;
 }
 
-static int _segtype_disp(struct report_handle *rh, struct field *field,
+static int _segtype_disp(struct report_handle *rh __attribute((unused)),
+			 struct field *field,
 			 const void *data)
 {
 	const struct lv_segment *seg = (const struct lv_segment *) data;
@@ -491,10 +494,9 @@ static int _origin_disp(struct report_handle *rh, struct field *field,
 			const void *data)
 {
 	const struct logical_volume *lv = (const struct logical_volume *) data;
-	struct lv_segment *snap_seg;
 
-	if ((snap_seg = find_cow(lv)))
-		return _string_disp(rh, field, &snap_seg->origin->name);
+	if (lv_is_cow(lv))
+		return _string_disp(rh, field, &origin_from_cow(lv)->name);
 
 	field->report_string = "";
 	field->sort_value = (const void *) field->report_string;
@@ -527,8 +529,7 @@ static int _lvname_disp(struct report_handle *rh, struct field *field,
 	char *repstr;
 	size_t len;
 
-	/* FIXME Remove need for snapshot special case */
-	if (lv->status & VISIBLE_LV || lv_is_cow(lv)) {
+	if (lv_is_visible(lv)) {
 		repstr = lv->name;
 		return _string_disp(rh, field, &repstr);
 	}
@@ -581,7 +582,7 @@ static int _size32_disp(struct report_handle *rh, struct field *field,
 	const char *disp;
 	uint64_t *sortval;
 
-	if (!*(disp = display_size(rh->cmd, (uint64_t) size, SIZE_UNIT))) {
+	if (!*(disp = display_size_units(rh->cmd, (uint64_t) size))) {
 		stack;
 		return 0;
 	}
@@ -609,7 +610,7 @@ static int _size64_disp(struct report_handle *rh, struct field *field,
 	const char *disp;
 	uint64_t *sortval;
 
-	if (!*(disp = display_size(rh->cmd, size, SIZE_UNIT))) {
+	if (!*(disp = display_size_units(rh->cmd, size))) {
 		stack;
 		return 0;
 	}
@@ -667,11 +668,10 @@ static int _chunksize_disp(struct report_handle *rh, struct field *field,
 			   const void *data)
 {
 	const struct lv_segment *seg = (const struct lv_segment *) data;
-	struct lv_segment *snap_seg;
 	uint64_t size;
 
-	if ((snap_seg = find_cow(seg->lv)))
-		size = (uint64_t) snap_seg->chunk_size;
+	if (lv_is_cow(seg->lv))
+		size = (uint64_t) find_cow(seg->lv)->chunk_size;
 	else
 		size = 0;
 
@@ -840,7 +840,6 @@ static int _snpercent_disp(struct report_handle *rh, struct field *field,
 			   const void *data)
 {
 	const struct logical_volume *lv = (const struct logical_volume *) data;
-	struct lv_segment *snap_seg;
 	struct lvinfo info;
 	float snap_percent;
 	uint64_t *sortval;
@@ -851,16 +850,15 @@ static int _snpercent_disp(struct report_handle *rh, struct field *field,
 		return 0;
 	}
 
-	if (!(snap_seg = find_cow(lv)) ||
-	    (lv_info(lv->vg->cmd, snap_seg->cow, &info, 0) && !info.exists)) {
+	if (!lv_is_cow(lv) ||
+	    (lv_info(lv->vg->cmd, lv, &info, 0) && !info.exists)) {
 		field->report_string = "";
 		*sortval = UINT64_C(0);
 		field->sort_value = sortval;
 		return 1;
 	}
 
-	if (!lv_snapshot_percent(snap_seg->cow, &snap_percent)
-	    || snap_percent < 0) {
+	if (!lv_snapshot_percent(lv, &snap_percent) || snap_percent < 0) {
 		field->report_string = "100.00";
 		*sortval = UINT64_C(100);
 		field->sort_value = sortval;
